<?php

namespace ahrefs\AhrefsSeo;

use Google_Client;
/**
 * Class for interacting with Bugsnag.
 */
class Ahrefs_Seo_Bugsnag {

	/**
	 * @var string
	 */
	private $api_key = '476ca398513aa4c0b66b02a9cbd0bed4';
	/**
	 * @var Ahrefs_Seo_Bugsnag
	 */
	private static $instance;
	public static function get() {
		if ( empty( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}
	public function create_client() {
		$client = \Bugsnag\Client::make( $this->api_key );
		$client->setAppVersion( AHREFS_SEO_VERSION );
		$client->setReleaseStage( AHREFS_SEO_RELEASE );
		try {
			$client->registerCallback( [ $this, 'report_callback' ] );
			$client->setStripPathRegex( sprintf( '/^%s[\\/\\\\]/', preg_quote( rtrim( ABSPATH, '\\/' ), '/' ) ) );
		} catch ( \Exception $e ) {
			$client->notifyException( $e );
		}
		return $client;
	}
	/**
	 * Callback for reports
	 *
	 * @param \Bugsnag\Report $report
	 * @return mixed
	 */
	public function report_callback( $report ) {
		try {
			// Callback. Do not use parameter types.
			if ( ! $report instanceof \Bugsnag\Report ) {
				return;
			}
			$frames              = $report->getStacktrace()->getFrames();
			$found_in_stacktrace = false;
			foreach ( $frames as &$frame ) {
				if ( false !== strpos( $frame['file'], 'ahrefs-seo' ) && false === strpos( $frame['file'], 'ahrefs-seo' . DIRECTORY_SEPARATOR . 'vendor' ) ) {
					$found_in_stacktrace = true;
					break;
				}
			}
			if ( ! $found_in_stacktrace ) {
				return false;
			}
			$original_error = $report->getOriginalError();
			if ( is_array( $original_error ) && isset( $original_error['code'] ) && isset( $original_error['file'] ) && E_DEPRECATED === $original_error['code'] && strpos( $original_error['file'], 'vendor' . DIRECTORY_SEPARATOR . 'google' ) ) {
				return false;
			}
			$tokens  = null;
			$list    = null;
			$version = null;
			try {
				$tokens    = Ahrefs_Seo_Analytics::OPTION_TOKENS;
				$analytics = Ahrefs_Seo_Analytics::get();
				$list      = $analytics->load_gsc_accounts_list( true ); // cached value.
				$version = class_exists( '\\GuzzleHttp\\Client' ) ? \GuzzleHttp\Client::VERSION : '';
			} catch ( \Exception $e ) {
				$report->setMetaData( [ 'error' => [ 'exception' => $e ] ] );
			} catch ( \Error $e ) {
				$report->setMetaData( [ 'error' => [ 'error' => $e ] ] );
			}
			$report->setMetaData(
				[
					'info' => [
						'google-client'  => Google_Client::LIBVER,
						'guzzle-client'  => $version,
						'db-tables'      => Ahrefs_Seo::CURRENT_TABLE_VERSION,
						'content-rules'  => Ahrefs_Seo::CURRENT_CONTENT_RULES,
						'transient-time' => Ahrefs_Seo::transient_time(),
						'gsc-list'       => wp_json_encode( $list ), // cached value.
						'domain'         => Ahrefs_Seo::get_current_domain(),
						'url'            => home_url(),
						'wpurl'          => site_url(),
						'wordpress'      => get_bloginfo( 'version' ),
						'abspath'        => ABSPATH,
						'multisite'      => is_multisite() ? 'Yes' : 'No',
						'blog'           => get_current_blog_id(),
						'uid'            => get_current_user_id(),
						'ga-options'     => ! is_null( $tokens ) ? get_option( $tokens, '' ) : null, // @phpstan-ignore-line
						'thread'         => Ahrefs_Seo::thread_id(),
						'seo-list'       => Ahrefs_Seo_Noindex::get_plugins_list(),
					],
				]
			);
			$report->setUser( [ 'id' => Ahrefs_Seo::get_current_domain() ] );
			if ( $original_error instanceof \Exception ) { // only for Exceptions, not for Errors.
				$this->set_grouping_hash( $report );
			}
		} catch ( \Exception $e ) {
			$report->addBreadcrumb( new \Bugsnag\Breadcrumbs\Breadcrumb( 'Error in callback: ' . (string) $e, \Bugsnag\Breadcrumbs\Breadcrumb::ERROR_TYPE ) );
		}
	}
	/**
	 * Set grouping hash for API request reports
	 *
	 * @since 0.8.0
	 *
	 * @param \Bugsnag\Report $report
	 * @return void
	 */
	public function set_grouping_hash( \Bugsnag\Report $report ) {
		try {
			$api_type = '';
			$methods  = [
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Analytics::get_clicks_and_impressions_by_urls' => 'gsc-keywords',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Analytics::get_position_fast' => 'gsc-position',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Analytics::get_visitors_by_page_ga4' => 'ga4-traffic',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Analytics::get_visitors_by_page_ga' => 'ga4-traffic',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Noindex::is_noindex'        => 'noindex',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Api::get_backlinks_count_by_url' => 'ahrefs-metrics_extended',
				'ahrefs\\AhrefsSeo\\Ahrefs_Seo_Api::get_subscription_info' => 'ahrefs-subscription_info',
			];
			$frames   = $report->getStacktrace()->getFrames();
			foreach ( $frames as &$frame ) {
				if ( false !== strpos( $frame['file'], 'ahrefs-seo' ) && isset( $frame['method'] ) && isset( $methods[ $frame['method'] ] ) ) {
					$api_type = $methods[ $frame['method'] ];
					break;
				}
			}
			if ( '' !== $api_type ) {
				$hash           = null;
				$original_error = $report->getOriginalError();
				if ( $original_error instanceof \Exception ) {
					$hash = $this->exception_description( $original_error );
				}
				if ( ! is_null( $hash ) ) {
					$report->setGroupingHash( "{$api_type}-{$hash}" );
				}
			}
		} catch ( \Exception $e ) {
			$report->addBreadcrumb( new \Bugsnag\Breadcrumbs\Breadcrumb( 'Error in grouping: ' . (string) $e, \Bugsnag\Breadcrumbs\Breadcrumb::ERROR_TYPE ) );
		}
	}
	protected function exception_description( \Exception $e ) {
		$hash = 'other';
		if ( $e instanceof \Google_Service_Exception ) {
			$hash = 'google-other';
			/** @var \Google_Service_Exception $e */
			$code  = $e->getCode();
			$error = $e->getErrors();
			if ( is_array( $error ) && isset( $error[0] ) && isset( $error[0]['reason'] ) ) {
				$hash = "google-{$code}-{$error[0]['reason']}";
			} elseif ( preg_match( '/<title>Error (\\d+)/', $e->getMessage(), $m ) ) {
				// parse html title.
				$hash = "google-html-{$m[1]}"; // example: 'google-html-502'.
			}
		} elseif ( $e instanceof \GuzzleHttp\Exception\ConnectException ) {
			/** @var \GuzzleHttp\Exception\ConnectException $e */
			$message = $e->getMessage();
			$hash    = 'guzzle-connect-other';
			if ( preg_match( '/^cURL error (\\d+):/', $message, $m ) ) {
				$hash = "guzzle-connect-curl-{$m[1]}"; // example: 'guzzle-connect-curl-28'.
			}
		} elseif ( $e instanceof \GuzzleHttp\Exception\RequestException ) {
			/** @var \GuzzleHttp\Exception\RequestException $e */
			$message = $e->getMessage();
			$hash    = 'guzzle-request-other';
			if ( preg_match( '/^cURL error (\\d+):/', $message, $m ) ) {
				$hash = "guzzle-request-curl-{$m[1]}"; // example: 'guzzle-request-curl-60'.
			} elseif ( preg_match( '/^(\\w+) error: `[^`]+` resulted in a `(\\d+) ([^`]+)`/', $message, $m ) ) {
				$what = strtolower( $m[1] ); // one of 'client' or 'server'.
				$code = (int) $m[2]; // http error code.
				$hash = "guzzle-{$what}-{$code}"; // example: 'guzzle-client-401' or 'guzzle-server-503'.
			}
		} elseif ( $e instanceof Ahrefs_Seo_Exception ) {
			/** @var Ahrefs_Seo_Exception $e */
			$hash           = 'ahrefs_exception';
			$original_error = $e->getPrevious();
			if ( $original_error instanceof \Exception ) {
				$hash .= '-' . $this->exception_description( $original_error );
			}
		}
		return $hash;
	}
}
