<?php
declare(strict_types=1);

namespace ahrefs\AhrefsSeo;

/**
 * Data keyword content storage
 *
 * @since 0.8.2
 */
class Data_Keyword_Content_Storage {
	/** @var string */
	private $content = '';

	/** @var int */
	private $count = 0;

	/**
	 * List of keyword items, ordered by featured desc.
	 *
	 * @var Data_Keyword_Tfidf[]
	 */
	private $keywords = [];

	/**
	 * Get words count in content
	 *
	 * @return int
	 */
	public function get_count() : int {
		return $this->count;
	}

	/**
	 * Set keywords
	 *
	 * @param array<string|int, float> $keywords_raw
	 * @return Data_Keyword_Content_Storage
	 */
	public function set_keywords( array $keywords_raw ) : self {
		$this->content  = '';
		$this->keywords = [];
		foreach ( $keywords_raw as $word => $featured ) {
			$this->keywords[] = new Data_Keyword_Tfidf( "$word", $featured );
		}
		return $this;
	}

	/**
	 * Get keywords
	 *
	 * @return Data_Keyword_Tfidf[] List of keyword items.
	 */
	public function get_keywords() : array {
		return $this->keywords;
	}

	/**
	 * Set content
	 *
	 * @param string $content
	 * @return Data_Keyword_Content_Storage
	 */
	public function set_content( string $content ) : self {
		$this->content = $content;
		$this->count   = strlen( $content ) ? substr_count( $content, ' ' ) + substr_count( $content, '|' ) + 1 : 0;
		return $this;
	}

	/**
	 * Get content
	 *
	 * @return string
	 */
	public function get_content() : string {
		return $this->content;
	}
}
